<?xml version="1.0" encoding="UTF-8"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:lexs="http://usdoj.gov/leisp/lexs/3.1" 
    xmlns:w="http://niem.gov/niem/wantlist/1" 
    exclude-result-prefixes="xs xsd" version="2.0">

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: David RR Webber, July, 2008 -->

    <!-- 

Purpose : Take a  CAM template and insert anonymous namespace prefix and URI
Original : 07/21/2008 
Initials : drrw 
CAM ver : 1.1 
Description : Part of a set of tools to ease alignment of XSD based validations and documentation 
and CAM templates. 
How to use :  Provide prefix to insert for anonymous namespace and the URI corresponding.
Updated - Inits - Change 
07/21/08 drrw Initial release 
-->
    <xsl:param name="anonymousNS"></xsl:param>
    <xsl:param name="URI"></xsl:param>
    
    <xsl:output indent="yes" omit-xml-declaration="yes"/>
    
    <xsl:variable name="version">1.0</xsl:variable>
   
    <xsl:template match="/">
       
        <xsl:element name="as:CAM" xpath-default-namespace="{$URI}">
            <xsl:copy-of select=".//namespace::*"/>
            <xsl:copy-of select="as:CAM/@*"/>
            <xsl:if test="$anonymousNS &gt; ' '">
                <xsl:namespace name="{$anonymousNS}"><xsl:value-of select="$URI"/></xsl:namespace>
            </xsl:if>
            <xsl:copy-of select="as:CAM/as:Header"></xsl:copy-of>
            <xsl:element name="as:AssemblyStructure">
                <xsl:element name="as:Structure">
                    <xsl:copy-of select="as:CAM/as:AssemblyStructure/as:Structure/@*"/>
                        <xsl:apply-templates select="as:CAM/as:AssemblyStructure/as:Structure/child::*" mode="structure"/>
                </xsl:element>
            </xsl:element>
            <xsl:apply-templates select="as:CAM/as:BusinessUseContext" mode="rules"/>
            <xsl:copy-of select="as:CAM/as:include"></xsl:copy-of>
            <xsl:copy-of select="as:CAM/as:Extension"></xsl:copy-of>
        </xsl:element>
       
    </xsl:template>  
  
    <xsl:template match="*" mode="structure">
        <xsl:variable name="this">
            <xsl:choose>
                <xsl:when test="contains(name(.),':')"><xsl:value-of select="name(.)"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat($anonymousNS,':',name(.))"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="nsURI">
            <xsl:choose>
                <xsl:when test="contains(name(.),':')"><xsl:value-of select="namespace-uri(.)"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="$URI"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:element name="{$this}" namespace="{$nsURI}">
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates select="child::*" mode="structure"/>
            <xsl:value-of select="if(text() &gt; ' ') then text() else ''"/>
        </xsl:element>
    </xsl:template>
  
   <xsl:template match="*" mode="rules">
       <xsl:element name="{name(.)}">
           <xsl:for-each select="@*">
               <xsl:choose>
                   <xsl:when test="name(.)='action'">
                       <xsl:choose>
                           <xsl:when test="contains(.,'setChoice')">
                               <xsl:variable name="xpath" select="substring-after(substring-before(.,'/*'),'(')"></xsl:variable>
                               <xsl:variable name="newPath"><xsl:call-template name="adjustPath"><xsl:with-param name="path" select="$xpath"/></xsl:call-template></xsl:variable>
                               <xsl:variable name="newaction" select="concat(substring-before(.,'('),'(',$newPath,'/*',substring-after(.,'/*'))"></xsl:variable>
                               <xsl:attribute name="action" select="$newaction"/>
                           </xsl:when>
                           <xsl:when test="contains(.,',')">
                               <xsl:variable name="xpath" select="substring-after(substring-before(.,','),'(')"></xsl:variable>
                               <xsl:variable name="newPath"><xsl:call-template name="adjustPath"><xsl:with-param name="path" select="$xpath"/></xsl:call-template></xsl:variable>
                               <xsl:variable name="newaction" select="concat(substring-before(.,'('),'(',$newPath,',',substring-after(.,','))"></xsl:variable>
                               <xsl:attribute name="action" select="$newaction"/>
                           </xsl:when>
                           <xsl:otherwise>
                               <xsl:variable name="xpath" select="substring-after(substring-before(.,')'),'(')"></xsl:variable>
                               <xsl:variable name="newPath"><xsl:call-template name="adjustPath"><xsl:with-param name="path" select="$xpath"/></xsl:call-template></xsl:variable>
                               <xsl:variable name="newaction" select="concat(substring-before(.,'('),'(',$newPath,')')"></xsl:variable>
                               <xsl:attribute name="action" select="$newaction"/>
                           </xsl:otherwise>
                       </xsl:choose>
                   </xsl:when>
                   <xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
               </xsl:choose>
               
           </xsl:for-each>
           <xsl:apply-templates select="child::*" mode="rules"/>
       </xsl:element>
   </xsl:template>
    
    <xsl:template name="adjustPath">
        <xsl:param name="path"/>
        <xsl:variable name="parts" select="tokenize($path,'/')"/>
        <xsl:variable name="newPath">/<xsl:for-each select="$parts">
                <xsl:choose>
                    <xsl:when test=".=''"></xsl:when>
                    <xsl:when test=".='/'">/</xsl:when>
                    <xsl:when test="contains(.,':')">/<xsl:value-of select="."/></xsl:when>
                    <xsl:when test="starts-with(.,'@')">/<xsl:value-of select="."/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="concat('/',$anonymousNS,':',.)"/></xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:variable>
        <xsl:value-of select="$newPath"/>
    </xsl:template>
  
</xsl:stylesheet>
