<?xml version="1.0" encoding="UTF-8"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:lexs="http://usdoj.gov/leisp/lexs/3.1" 
    xmlns:w="http://niem.gov/niem/wantlist/1" 
    exclude-result-prefixes="xs xsd" version="2.0">

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: David RR Webber, April, 2008 -->

    <!-- 

Purpose : Take any XML instance and applies CAM wantlist to it
Original : 04/22/2008 
Initials : drrw 
CAM ver : 1.1 
Description : Part of a set of tools to ease alignment of XSD based validations and documentation 
and CAM templates. 
Updated - Inits - Change 
04/22/08 drrw Initial release 
    -->
    <xsl:param name="wantlistURL">EDXL-wantlist-result.xml</xsl:param>
    <xsl:param name="showXcl" as="xsd:boolean">true</xsl:param>
    
    <xsl:output indent="yes"/>
    
    <xsl:variable name="wanted" select="document($wantlistURL)"/>

    <xsl:template match="/">

        <xsl:message><xsl:value-of select="concat('wantlist rules=',count($wanted/w:WantList/descendant::*))"></xsl:value-of></xsl:message>
        
        <xsl:variable name="ns-prefix">
            <xsl:value-of select="./@targetNamespace"/>
        </xsl:variable>              
            
            <xsl:for-each select="*">
                <!-- parses the entire XML input -->
                <xsl:copy>
                    <xsl:copy-of select=".//namespace::*"/>
                    <xsl:copy-of select="./@*"/>
                    <xsl:apply-templates mode="map" select="*"/>                  
                </xsl:copy>
            </xsl:for-each>

    </xsl:template>
    
    <xsl:template match="node()/child::*" mode="map">
        <xsl:variable name="thisGrp" select="concat(name(../.),'\',name(.))"/>
        <xsl:variable name="XcludeGrp">
            <xsl:choose>
                <xsl:when test="$wanted/w:WantList//Group/@w:n=$thisGrp">
                    <xsl:variable name="chkRoot" select="$wanted/w:WantList//Group[@w:n=$thisGrp]"/>
                    <xsl:variable name="thisRoot" select="$chkRoot/../../@w:n[1]"/>
                    <xsl:variable name="action" select="$wanted/w:WantList//Group[@w:n=$thisGrp]/@w:isXclude"/>
                    <xsl:value-of select="$action[1]"/>
                </xsl:when>
                <xsl:otherwise>0</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="not($XcludeGrp='0') and $showXcl"><xsl:comment>Excluded Group: <xsl:value-of select="name(.)"/></xsl:comment></xsl:if>
        <xsl:if test="$XcludeGrp='0'">
            <xsl:copy>
               
            <xsl:variable name="thisElement" select="node()"></xsl:variable>
            <!-- xsl:element name="{local-name(.)}" -->
                <!-- xsl:comment><xsl:value-of select="$action[1]"/> - <xsl:value-of  select="name(../.)"/> - [<xsl:value-of select="$thisRoot[1]"></xsl:value-of>]</xsl:comment></xsl:if -->
                <!-- xsl:apply-templates select="./@*" mode="map"/-->
                
            <xsl:for-each select="./@*">
                <xsl:variable name="this" select="concat(name(../../.),'\',name(../.),'\@',name(.))"/>
                <xsl:variable name="Xclude">
                    <xsl:choose>
                        <xsl:when test="$wanted/w:WantList//Attribute/@w:n=$this">
                            <xsl:variable name="chkRoot" select="$wanted/w:WantList//Attribute[@w:n=$this]"/>
                            <xsl:variable name="thisRoot" select="$chkRoot/../../@w:n[1]"/>
                            <xsl:variable name="action" select="$wanted/w:WantList//Attribute[@w:n=$this]/@w:isXclude"/>
                            <xsl:value-of select="$action[1]"/>
                        </xsl:when>
                        <xsl:otherwise>0</xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:if test="$Xclude='0'">
                    <xsl:copy-of select="."/>
                </xsl:if>
            </xsl:for-each>
            <xsl:copy-of select="$thisElement[1]"/>
                
            <xsl:for-each select="*">
                <xsl:choose>
                    <xsl:when test="count(./descendant::node()) > 1">
                         <xsl:apply-templates mode="map" select="."/>
                         <xsl:apply-templates mode="map" select="./@*"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:variable name="this" select="concat(name(../../.),'\',name(../.),'\',name(.))"/>
                        <xsl:variable name="Xclude">
                        <xsl:choose>
                            <xsl:when test="$wanted/w:WantList//Element/@w:n=$this">
                                <xsl:variable name="chkRoot" select="$wanted/w:WantList//Element[@w:n=$this]"/>
                                <xsl:variable name="thisRoot" select="$chkRoot/../../@w:n[1]"/>
                                <xsl:variable name="action" select="$wanted/w:WantList//Element[@w:n=$this]/@w:isXclude"/>
                                <xsl:value-of select="$action[1]"/>
                            </xsl:when>
                            <xsl:otherwise>0</xsl:otherwise>
                        </xsl:choose>
                        </xsl:variable>
                        <xsl:if test="not($Xclude='0') and $showXcl"><xsl:comment>Excluded: <xsl:value-of select="name(.)"/></xsl:comment></xsl:if>
                       <xsl:if test="$Xclude='0'">
                        <!-- output element and attributes -->
                           <xsl:copy>
                                <!-- check attribute not excluded -->
                               <xsl:for-each select="@*">
                                   <xsl:variable name="this" select="concat(name(../../.),'\',name(../.),'\@',name(.))"/>
                                   <xsl:variable name="Xclude">
                                       <xsl:choose>
                                           <xsl:when test="$wanted/w:WantList//Attribute/@w:n=$this">
                                               <xsl:variable name="chkRoot" select="$wanted/w:WantList//Attribute[@w:n=$this]"/>
                                               <xsl:variable name="thisRoot" select="$chkRoot/../../@w:n[1]"/>
                                               <xsl:variable name="action" select="$wanted/w:WantList//Attribute[@w:n=$this]/@w:isXclude"/>
                                               <xsl:value-of select="$action[1]"/>
                                           </xsl:when>
                                           <xsl:otherwise>0</xsl:otherwise>
                                       </xsl:choose>
                                   </xsl:variable>
                                   <xsl:if test="$Xclude='0'">
                                       <xsl:copy-of select="."/>
                                   </xsl:if>
                               </xsl:for-each>
                               <xsl:copy-of select="node()"/>
                           </xsl:copy>
                       </xsl:if>
                    </xsl:otherwise>
                </xsl:choose>   
            </xsl:for-each>
            <!-- /xsl:element -->
                
             </xsl:copy>
        </xsl:if>
    </xsl:template>   
    
    <xsl:template match="attribute::node()" mode="map">
        <!-- xsl:message><xsl:value-of select="concat('attrib: ',name(../.),'\@',name())"/></xsl:message -->
    </xsl:template>

</xsl:stylesheet>
